' Common objects
Dim fso, wso
Set fso = CreateObject("Scripting.FileSystemObject")
Set wso = CreateObject ("WScript.Shell")

' Common functions
' Registry functions
Function ReadRegistryKey(key, ByRef value)
	On Error Resume Next
	Dim wso
	Set wso=CreateObject ("WSCript.Shell")	
	value = wso.RegRead(key)
	If Err.Number <> 0 Then
		Err.Clear
		value = ""
		ReadRegistryKey = 1
		Exit Function
	End If
	ReadRegistryKey = 0
End Function

Sub WriteRegistryKey(key, value)
	WriteRegistryKeyEx key, value, "REG_SZ"
End Sub

Sub WriteRegistryKeyEx(key, value, keyType)
	On Error Resume Next
	
	LogProgress vbTab & "Writing registry value '" & value & "' to key '" & key & "'"
	
	Dim wso
	Set wso=CreateObject ("WSCript.Shell")	
	wso.RegWrite key, value, keyType
	If Err.Number <> 0 Then	FailedQuit "Failed to write registry value '" & value & "' to key '" & key & "'"
End Sub

' Logging functions
Function FailedQuit(msg)
	LogError msg
	WScript.Quit -1
End Function

Function LogProgress(msg)
	WScript.Echo msg
End Function

Function LogError(msg)
	WScript.StdErr.Write "[ERROR] " & msg & vbCrLf
End Function

' Exec program
Function ExecProgram(cmd, strInput, ByRef strOutput, ByRef strError)
	On Error Resume Next

	LogProgress vbTab & "Executing '" & cmd & "'"
	
	Dim oExec
	Set oExec = wso.Exec(cmd)

	If Err.number <> 0 Then
		strError = Err.Description
		ExecProgram = Err.number
		Exit Function
	End If

	oExec.StdIn.Write strInput
	oExec.StdIn.Close
	
	strOutput = oExec.StdOut.ReadAll
	strError = oExec.StdErr.ReadAll
	
	Do 
		WScript.Sleep 100
		If Not oExec.StdOut.AtEndOfStream Then
			strOutput = strOutput & oExec.StdOut.ReadAll
		End If
		
		If Not oExec.StdErr.AtEndOfStream Then
			strError = strError & oExec.StdErr.ReadAll
		End If
	Loop Until oExec.Status <> 0

	oExec.StdOut.Close
   	oExec.StdErr.Close
    
	ExecProgram = oExec.ExitCode
End Function 

' File functions
Function GetTempFileName
	Dim tfolder, tname
	Const TemporaryFolder = 2
	Set tfolder = fso.GetSpecialFolder(TemporaryFolder)
	GetTempFileName = tfolder.Path & "\" & fso.GetTempName
End Function

' Service functions
Function SetServiceAutostart(name)
	Dim Locator, Services, Service, ret

	LogProgress "Changing service '" & name & "' startmode"

	Const COMPUTER = "."
	
	Set Locator = CreateObject("WbemScripting.SWbemLocator")
	Set Services = Locator.ConnectServer(".")
	Set Service = Services.Get("Win32_Service='" & name & "'")

	ret = Service.Change(, , , , "Automatic")
	If ret <> 0 Then 
		LogError "Failed to change service '" & name & "' start mode"
		SetServiceAutostart = ret
		Exit Function
	End If


	LogProgress vbTab & "Starting service " & name
	Service.StartService
	WaitService Services, Service, name, 1 * 60, "Running"
	If Service.State <> "Running" Then
		LogError "Failed to start service '" & name & "' (" & Service.State & ")"
		SetServiceAutostart = ret
		Exit Function
	End If
End Function

Sub WaitService(Services, ServiceObject, name, seconds, state)
	Dim startTime
	startTime = Timer
	Do While (Timer - startTime < seconds) AND (ServiceObject.State <> state)
		WScript.Sleep 1000
		Set ServiceObject = Services.Get("Win32_Service='" & name & "'")
	Loop
End Sub


' PVA-specific functions
Function SaveUnattendedParameters(iisUnattendedInstall)

	' Save sysocmgr unattended parameters
	Dim fo
	Set fo = fso.OpenTextFile(iisUnattendedInstall, 8, True)
	fo.WriteLine "[Components]"
	fo.WriteLine "iis_www=on"
	fo.WriteLine "aspnet=on"
	fo.WriteLine "iis_asp=on"

	fo.Close

End Function

Function RunConfigurator(hinttype)
	Dim cmd, output, error, ret

	cmd = Mid(Wscript.ScriptFullName, 1, InStrRev(Wscript.ScriptFullName, "\")) & "pva-cc-config.exe " & hinttype

	ret = ExecProgram(cmd, "", output, error)
	If ret <> 0 Then 
		LogProgress output
		LogError "Failed exec '" & cmd & "': " & vbCrLf & error
		RunConfigurator = ret
		Exit Function
	Else
		LogProgress output
		LogProgress error
		LogProgress "'" & cmd & "' finished successfully"
	End If

End Function

' CC hints implementation
' Install additional IIS components and run pva-cp-config preInstall
Function PreInstall

	' Get temp file name	
	Dim iisUnattendedInstall
	iisUnattendedInstall = GetTempFileName

	' Save sysocmgr unattended parameters
	LogProgress "Saving sysocmgr unattended parameters to file '" & iisUnattendedInstall & "'"
	On Error Resume Next
	SaveUnattendedParameters iisUnattendedInstall
	If Err Then
		LogError "Failed to save sysocmgr unattended parameters - " & Err.Description
		PreInstall = Err.number
		Exit Function
	End If
	On Error Goto 0

	' Set windows distributive path
	If WScript.Arguments.Named.Exists("os_distributive_path") Then
		Const sourcePathKey = "HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\Setup\SourcePath"
		Const servicePackSourcePathKey = "HKEY_LOCAL_MACHINE\SOFTWARE\Microsoft\Windows\CurrentVersion\Setup\ServicePackSourcePath"
		Dim sourcePath
		sourcePath = WScript.Arguments.Named.Item("os_distributive_path")
		LogProgress "Changing default Windows Distributive source path to '" & sourcePath & "'"
		WriteRegistryKey sourcePathKey, sourcePath
		WriteRegistryKey servicePackSourcePathKey, sourcePath
        End If

	' Exec sysocmgr
	LogProgress "Installing additional IIS components"
	Dim ret, output, error
	ret = ExecProgram("Sysocmgr.exe /x /i:sysoc.inf /u:""" & iisUnattendedInstall & """", "", output, error)

	fso.DeleteFile iisUnattendedInstall, True

	If ret <> 0 Then 
		LogError "Failed to install IIS components - " & error
		PreInstall = ret
		Exit Function
	End If

	' Modify services configs
	On Error Resume Next
	ret = SetServiceAutostart("IISADMIN")
	If Err Then
		LogError "Failed to modify IISADMIN service start mode - " & Err.Description
		PreInstall = Err.number
		Exit Function
	End If
	If ret <> 0 Then 
		LogError "Failed to modify IISADMIN service start mode"
		PreInstall = ret
		Exit Function
	End If
	
	ret = SetServiceAutostart("W3SVC")
	If Err Then
		LogError "Failed to modify W3SVC service start mode - " & Err.Description
		PreInstall = Err.number
		Exit Function
	End If
	If ret <> 0 Then 
		LogError "Failed to modify W3SVC service start mode"
		PreInstall = ret
		Exit Function
	End If

	' Install ASP.NET
	LogProgress "Installing ASP.NET"
	Dim cmd
	cmd = fso.GetSpecialFolder(0) & "\Microsoft.NET\Framework\v1.1.4322\aspnet_regiis -i -enable"
	ret = ExecProgram(cmd, "", output, error)

	If ret <> 0 Then
		LogError "Failed to install ASP.NET - " & error
		PreInstall = ret
		Exit Function
	End If

	ret = RunConfigurator("preInstall")
	If ret <> 0 Then
		LogError "Preinstall configuration failed"
		PreInstall = ret
	End If
End Function

Function PreUpgrade
	Dim ret
	ret = RunConfigurator("preUpgrade")
	If ret <> 0 Then
		LogError "Preupgrade configuration failed"
		PreUpgrade = ret
	End If
End Function

Function PreUninstall
	Dim ret
	ret = RunConfigurator("preUninstall")
	If ret <> 0 Then
		LogError "Preuninstall configuration failed"
		PreUninstall = ret
	End If
End Function

Function ExecVZAHelper(what)
	Dim buf, cmd, output, error, ret
	
	ret = ReadRegistryKey("HKEY_LOCAL_MACHINE\SOFTWARE\Parallels\PVA\mn\INSTALLDIR", buf) 
	
	If ret <> 0 Then
		LogError "PVA Management node not found."
		ExecVZAHelper = ret
		Exit Function
	End If

	cmd = buf & "\bin\vzlInstallHelper " & what

	ret = ExecProgram(cmd, "", output, error)
	If ret <> 0 and ret <> 238 and ret <> 252 Then 
		LogProgress output
		LogError "Failed exec '" & cmd & "': " & vbCrLf & error
		ExecVZAHelper = ret
		Exit Function
	Else
		LogProgress output
		LogProgress error
		LogProgress "'" & cmd & "' finished successfully"
		ExecVZAHelper = 0
	End If
End Function

Function CreateGroup
	Dim ret
	ret = ExecVZAHelper("addGroup ""PVA Control Center Users""")
	If ret <> 0 Then
		CreateGroup = ret
	Else
		CreateGroup = ExecVZAHelper("addRoleAssignment ""PVA User"" ""PVA Control Center Users""")
	End If
End Function
		

Function PostInstall
	Dim ret
	ret = CreateGroup
	If ret = 0 Then
		ret = RunConfigurator("PostInstall")
	End If
	If ret <> 0 Then
		LogError "Postinstall configuration failed"
		PostInstall = ret
	End If
End Function

Function PostUpgrade
	Dim ret
	ret = CreateGroup
	If ret = 0 Then
		ret = RunConfigurator("PostUpgrade")
	End If
	If ret <> 0 Then
		LogError "Postupgrade configuration failed"
		PostUpgrade = ret
	End If
End Function

Function PostUninstall
	Dim ret
	ret = RunConfigurator("PostUninstall")
	If ret <> 0 Then
		LogError "Postuninstall configuration failed"
		PostUninstall = ret
	End If
End Function
